@@ -5,7 +5,7 @@ use warnings;
 use Carp;
 use Cwd 'realpath';
 
-our $VERSION   = '1.3072';
+our $VERSION   = '1.3090';
 our $AUTHORITY = 'SUKRIA';
 
 use Dancer::App;
@@ -26,6 +26,14 @@ use Dancer::Session;
 use Dancer::SharedData;
 use Dancer::Handler;
 use Dancer::MIME;
+use Dancer::Exception qw(:all);
+
+use Dancer::Continuation::Halted;
+use Dancer::Continuation::Route::Forwarded;
+use Dancer::Continuation::Route::Passed;
+use Dancer::Continuation::Route::ErrorSent;
+use Dancer::Continuation::Route::FileSent;
+use Dancer::Continuation::Route::Templated;
 
 use File::Spec;
 
@@ -67,6 +75,7 @@ our @EXPORT    = qw(
   params
   pass
   path
+  patch
   post
   prefix
   push_header
@@ -99,10 +108,26 @@ our @EXPORT    = qw(
 
 # Dancer's syntax
 
-sub after           { Dancer::Hook->new('after', @_) }
+sub after           {
+    Dancer::Deprecation->deprecated(reason => "use hooks!",
+                                    version => '1.3080',
+                                    fatal => 0);
+    Dancer::Hook->new('after', @_);
+}
+sub before          {
+    Dancer::Deprecation->deprecated(reason => "use hooks!",
+                                    version => '1.3080',
+                                    fatal => 0);
+    Dancer::Hook->new('before', @_);
+}
+sub before_template {
+    Dancer::Deprecation->deprecated(reason => "use hooks!",
+                                    version => '1.3080',
+                                    fatal => 0);
+    Dancer::Hook->new('before_template', @_);
+}
+
 sub any             { Dancer::App->current->registry->any_add(@_) }
-sub before          { Dancer::Hook->new('before', @_) }
-sub before_template { Dancer::Hook->new('before_template', @_) }
 sub captures        { Dancer::SharedData->request->params->{captures} }
 sub cookie          { Dancer::Cookies->cookie( @_ ) }
 sub cookies         { Dancer::Cookies->cookies }
@@ -115,13 +140,19 @@ sub dirname         { Dancer::FileUtils::dirname(@_) }
 sub engine          { Dancer::Engine->engine(@_) }
 sub error           { goto &Dancer::Logger::error }
 sub false           { 0 }
-sub forward         { Dancer::SharedData->response->forward(@_) }
+sub forward         { Dancer::SharedData->response->forward(@_);
+                      # throw a special continuation exception
+                      Dancer::Continuation::Route::Forwarded->new->throw;
+                    }
 sub from_dumper     { Dancer::Serializer::Dumper::from_dumper(@_) }
 sub from_json       { Dancer::Serializer::JSON::from_json(@_) }
 sub from_xml        { Dancer::Serializer::XML::from_xml(@_) }
 sub from_yaml       { Dancer::Serializer::YAML::from_yaml(@_) }
 sub get             { map { my $r = $_; Dancer::App->current->registry->universal_add($r, @_) } qw(head get)  }
-sub halt            { Dancer::SharedData->response->halt(@_) }
+sub halt            { Dancer::SharedData->response->halt(@_);
+                      # throw a special continuation exception
+                      Dancer::Continuation::Halted->new->throw;
+                    }
 sub header          { goto &headers }
 sub push_header     { Dancer::SharedData->response->push_header(@_); }
 sub headers         { Dancer::SharedData->response->headers(@_); }
@@ -129,19 +160,23 @@ sub hook            { Dancer::Hook->new(@_) }
 sub layout          {
     Dancer::Deprecation->deprecated(reason => "use 'set layout => \"value\"'",
                                     version => '1.3050',
-                                    fatal => 0);
-    set(layout => shift) }
+                                    fatal => 1);
+}
 sub load            { require $_ for @_ }
 sub load_app        { goto &_load_app } # goto doesn't add a call frame. So caller() will work as expected
 sub logger          {
-    Dancer::Deprecation->deprecated(reason => "use 'set logger'",fatal => 0,version=>'1.3050');
-    set(logger => @_)
+    Dancer::Deprecation->deprecated(reason => "use 'set logger => \"value\"'",
+                                    fatal => 1,version=>'1.3050');
 }
 sub mime            { Dancer::MIME->instance() }
 sub options         { Dancer::App->current->registry->universal_add('options', @_) }
 sub params          { Dancer::SharedData->request->params(@_) }
 sub param           { params->{$_[0]} }
-sub pass            { Dancer::SharedData->response->pass(1) }
+sub pass            { Dancer::SharedData->response->pass(1);
+                      # throw a special continuation exception
+                      Dancer::Continuation::Route::Passed->new->throw;
+                    }
+sub patch            { Dancer::App->current->registry->universal_add('patch', @_) }
 sub path            { Dancer::FileUtils::path(@_) }
 sub post            { Dancer::App->current->registry->universal_add('post', @_) }
 sub prefix          { Dancer::App->current->set_prefix(@_) }
@@ -149,8 +184,16 @@ sub put             { Dancer::App->current->registry->universal_add('put',     @
 sub redirect        { goto &_redirect }
 sub render_with_layout { Dancer::Template::Abstract->_render_with_layout(@_) }
 sub request         { Dancer::SharedData->request }
-sub send_error      { Dancer::Error->new(message => $_[0], code => $_[1] || 500)->render() }
-sub send_file       { goto &_send_file }
+sub send_error      { Dancer::Continuation::Route::ErrorSent->new(
+                          return_value => Dancer::Error->new(
+                              message => $_[0],
+                              code => $_[1] || 500)->render()
+                      )->throw }
+#sub send_file       { goto &_send_file }
+sub send_file       { Dancer::Continuation::Route::FileSent->new(
+                          return_value => _send_file(@_)
+                      )->throw
+                    }
 sub set             { goto &setting }
 sub set_cookie      { Dancer::Cookies->set_cookie(@_) }
 sub setting         { Dancer::App->applications ? Dancer::App->current->setting(@_) : Dancer::Config::setting(@_) }
@@ -158,7 +201,9 @@ sub session         { goto &_session }
 sub splat           { @{ Dancer::SharedData->request->params->{splat} || [] } }
 sub start           { goto &_start }
 sub status          { Dancer::SharedData->response->status(@_) }
-sub template        { Dancer::Template::Abstract->template(@_) }
+sub template        { Dancer::Continuation::Route::Templated->new(
+                          return_value => Dancer::Template::Abstract->template(@_)
+                      )->throw }
 sub to_dumper       { Dancer::Serializer::Dumper::to_dumper(@_) }
 sub to_json         { Dancer::Serializer::JSON::to_json(@_) }
 sub to_xml          { Dancer::Serializer::XML::to_xml(@_) }
@@ -230,7 +275,7 @@ sub _load_app {
     # load the application
     _init_script_dir($script);
     my ($res, $error) = Dancer::ModuleLoader->load($app_name);
-    $res or croak "unable to load application $app_name : $error";
+    $res or raise core => "unable to load application $app_name : $error";
 
     # restore the main application
     Dancer::App->set_running_app('main');
@@ -258,10 +303,11 @@ sub _init_script_dir {
 
     my $LAYOUT_PRE_DANCER_1_2 = 1;
 
-    # in bin/ or public/ we need to go one level upper to find the appdir
+    # in bin/ or public/ or t/ we need to go one level upper to find the appdir
     $LAYOUT_PRE_DANCER_1_2 = 0
       if ($script_dirs[$#script_dirs - 1] eq 'bin')
-      or ($script_dirs[$#script_dirs - 1] eq 'public');
+      or ($script_dirs[$#script_dirs - 1] eq 'public')
+      or ($script_dirs[$#script_dirs - 1] eq 't');
 
     my $appdir = $ENV{DANCER_APPDIR} || (
           $LAYOUT_PRE_DANCER_1_2
@@ -285,7 +331,7 @@ sub _init_script_dir {
       || Dancer::FileUtils::path($appdir, 'views'));
 
     my ($res, $error) = Dancer::ModuleLoader->use_lib(Dancer::FileUtils::path($appdir, 'lib'));
-    $res or croak "unable to set libdir : $error";
+    $res or raise core => "unable to set libdir : $error";
 }
 
 
@@ -308,7 +354,7 @@ sub _redirect {
 
 sub _session {
     engine 'session'
-      or croak "Must specify session engine in settings prior to using 'session' keyword";
+      or raise core => "Must specify session engine in settings prior to using 'session' keyword";
       @_ == 0 ? Dancer::Session->get
     : @_ == 1 ? Dancer::Session->read(@_)
     :           Dancer::Session->write(@_);
@@ -316,10 +362,17 @@ sub _session {
 
 sub _send_file {
     my ($path, %options) = @_;
+    my $env = Dancer::SharedData->request->env;
 
     my $request = Dancer::Request->new_for_request('GET' => $path);
     Dancer::SharedData->request($request);
 
+    # if you asked for streaming but it's not supported in PSGI
+    if ( $options{'streaming'} && ! $env->{'psgi.streaming'} ) {
+        # TODO: throw a fit (AKA "exception") or a Dancer::Error?
+        raise core => 'Sorry, streaming is not supported on this server.';
+    }
+
     if (exists($options{content_type})) {
         $request->content_type($options{content_type});
     }
@@ -339,11 +392,55 @@ sub _send_file {
             $resp = Dancer::Renderer->get_file_response();
         }
     }
+
     if (exists($options{filename})) {
         $resp->push_header('Content-Disposition' => 
             "attachment; filename=\"$options{filename}\""
         );
     }
+
+    if ( $options{'streaming'} ) {
+        # handle streaming
+        $resp->streamed( sub {
+            my ( $status, $headers ) = @_;
+            my %callbacks = defined $options{'callbacks'} ?
+                            %{ $options{'callbacks'} }    :
+                            {};
+
+            return sub {
+                my $respond = shift;
+                exists $callbacks{'override'}
+                    and return $callbacks{'override'}->( $respond, $resp );
+
+                # get respond callback and set headers, get writer in return
+                my $writer = $respond->( [
+                    $status,
+                    $headers,
+                ] );
+
+                # get content from original response
+                my $content = $resp->content;
+
+                exists $callbacks{'around'}
+                    and return $callbacks{'around'}->( $writer, $content );
+
+                if ( ref $content ) {
+                    my $bytes = $options{'bytes'} || '43008'; # 42K (dams)
+                    my $buf;
+                    while ( ( my $read = sysread $content, $buf, $bytes ) != 0 ) {
+                        if ( exists $callbacks{'around_content'} ) {
+                            $callbacks{'around_content'}->( $writer, $buf );
+                        } else {
+                            $writer->write($buf);
+                        }
+                    }
+                } else {
+                    $writer->write($content);
+                }
+            };
+        } );
+    }
+
     return $resp if $resp;
 
     Dancer::Error->new(
@@ -436,7 +533,7 @@ your app.  You can control the exporting through the normal
 L<Exporter> mechanism.  For example:
 
     # Just export the route controllers
-    use Dancer qw(before after get post);
+    use Dancer qw(before after get post put patch);
 
     # Export everything but pass to avoid clashing with Test::More
     use Test::More;
@@ -496,11 +593,13 @@ Add a hook at the B<after> position:
 
     after sub {
         my $response = shift;
-        # do something with request
+        # do something with response, e.g.:
+        $response->header('X-Beer' => 'Yes please');
     };
 
 The anonymous function which is given to C<after> will be executed after
-having executed a route.
+having executed a route, but before the response is returned (so you can modify
+the response here, if you need to)..
 
 You can define multiple after filters, using the C<after> helper as
 many times as you wish; each filter will be executed, in the order you added
@@ -697,20 +796,21 @@ reached. If it was a B<GET>, it will remain a B<GET>.
 
 Broader functionality might be added in the future.
 
-It is important to note that issuing a forward by itself does not exit and
-forward immediately, forwarding is deferred until after the current route
-or filter has been processed. To exit and forward immediately, use the return
-function, e.g.
+B<WARNING> : Issuing a forward immediately exits the current route,
+and perform the forward. Thus, any code after a forward is ignored, until the
+end of the route. e.g.
 
     get '/some/path => sub {
         if ($condition) {
-            return forward '/articles/$article_id';
+            forward '/articles/$article_id';
+            # The following code is never executed
+            do_stuf();
         }
 
         more_stuff();
     };
 
-You probably always want to use C<return> with forward.
+So it's not necessary anymore to use C<return> with forward.
 
 Note that forward doesn't parse GET arguments. So, you can't use
 something like:
@@ -768,7 +868,9 @@ renders the response immediately:
 
     before sub {
         if ($some_condition) {
-            return halt("Unauthorized");
+            halt("Unauthorized");
+            # This code is not executed :
+            do_stuff();
         }
     };
 
@@ -776,6 +878,10 @@ renders the response immediately:
         "hello there";
     };
 
+B<WARNING> : Issuing a halt immediately exits the current route, and perform
+the halt. Thus, any code after a halt is ignored, until the end of the route.
+So it's not necessary anymore to use C<return> with halt.
+
 =head2 headers
 
 Adds custom headers to responses:
@@ -958,9 +1064,13 @@ by the layout
 
 =item after
 
-This is an alias for 'after'.
+This is an alias for C<after>.
 
-This hook receives as argument a L<Dancer::Response> object.
+This hook runs after a request has been processed, but before the response is
+sent.
+
+It receives a L<Dancer::Response> object, which it can modify
+if it needs to make changes to the response which is about to be sent.
 
   hook after => sub {
     my $response = shift;
@@ -980,6 +1090,26 @@ This hook receives as argument a L<Dancer::Response> object.
     my $response = shift;
   };
 
+=item on_handler_exception
+
+This hook is called when an exception has been caught, at the handler level,
+just before creating and rendering L<Dancer::Error>. This hook receives as
+argument a L<Dancer::Exception> object.
+
+  hook on_handler_exception => sub {
+    my $exception = shift;
+  };
+
+=item on_route_exception
+
+This hook is called when an exception has been caught, at the route level, just
+before rethrowing it higher. This hook receives the exception as argument. It
+can be a Dancer::Exception, or a string, or whatever was used to C<die>.
+
+  hook on_route_exception => sub {
+    my $exception = shift;
+  };
+
 =back
 
 =head2 layout
@@ -990,7 +1120,7 @@ This method is deprecated. Use C<set>:
 
 =head2 logger
 
-Deprecated. Use C<<set logger => 'console'>> to change current logger engine.
+Deprecated. Use C<<set logger =E<gt> 'console'>> to change current logger engine.
 
 =head2 load
 
@@ -1020,10 +1150,6 @@ To load multiple apps repeat load_app:
 The old way of loading multiple apps in one go (load_app 'one', 'two';) is
 deprecated.
 
-=head2 mime_type
-
-Deprecated. See L</mime>.
-
 =head2 mime
 
 Shortcut to access the instance object of L<Dancer::MIME>. You should
@@ -1073,15 +1199,38 @@ I<This method should be called from a route handler>.
 Tells Dancer to pass the processing of the request to the next
 matching route.
 
-You should always C<return> after calling C<pass>:
+B<WARNING> : Issuing a pass immediately exits the current route, and perform
+the pass. Thus, any code after a pass is ignored, until the end of the route.
+So it's not necessary anymore to use C<return> with pass.
 
     get '/some/route' => sub {
         if (...) {
             # we want to let the next matching route handler process this one
-            return pass();
+            pass(...);
+            # This code will be ignored
+            do_stuff();
         }
     };
 
+=head2 patch
+
+Defines a route for HTTP B<PATCH> requests to the given URL:
+
+    patch '/resource' => sub { ... };
+
+(C<PATCH> is a relatively new and not-yet-common HTTP verb, which is intended to
+work as a "partial-PUT", transferring just the changes; please see
+L<http://tools.ietf.org/html/rfc5789|RFC5789> for further details.)
+
+Please be aware that, if you run your app in standalone mode, C<PATCH> requests
+will not reach your app unless you have a new version of L<HTTP::Server::Simple>
+which accepts C<PATCH> as a valid verb.  The current version at time of writing,
+C<0.44>, does not.  A pull request has been submitted to add this support, which
+you can find at:
+
+L<https://github.com/bestpractical/http-server-simple/pull/1>
+
+
 =head2 path
 
 Concatenates multiple paths together, without worrying about the underlying
@@ -1238,10 +1387,18 @@ Returns a HTTP error.  By default the HTTP code returned is 500:
         }
     }
 
-This will not cause your route handler to return immediately, so be careful that
-your route handler doesn't then override the error.  You can avoid that by
-saying C<return send_error(...)> instead.
+B<WARNING> : Issuing a send_error immediately exits the current route, and perform
+the send_error. Thus, any code after a send_error is ignored, until the end of the route.
+So it's not necessary anymore to use C<return> with send_error.
 
+    get '/some/route' => sub {
+        if (...) {
+            # we want to let the next matching route handler process this one
+            send_error(..);
+            # This code will be ignored
+            do_stuff();
+        }
+    };
 
 =head2 send_file
 
@@ -1253,6 +1410,95 @@ the C<system_path> option (see below).
         return send_file(params->{file});
     }
 
+B<WARNING> : Issuing a send_file immediately exits the current route, and perform
+the send_file. Thus, any code after a send_file is ignored, until the end of the route.
+So it's not necessary anymore to use C<return> with send_file.
+
+    get '/some/route' => sub {
+        if (...) {
+            # we want to let the next matching route handler process this one
+            send_file(...);
+            # This code will be ignored
+            do_stuff();
+        }
+    };
+
+Send file supports streaming possibility using PSGI streaming. The server should
+support it but normal streaming is supported on most, if not all.
+
+    get '/download/:file' => sub {
+        return send_file( params->{file}, streaming => 1 );
+    }
+
+You can control what happens using callbacks.
+
+First, C<around_content> allows you to get the writer object and the chunk of
+content read, and then decide what to do with each chunk:
+
+    get '/download/:file' => sub {
+        return send_file(
+            params->{file},
+            streaming => 1,
+            callbacks => {
+                around_content => sub {
+                    my ( $writer, $chunk ) = @_;
+                    $writer->write("* $chunk");
+                },
+            },
+        );
+    }
+
+You can use C<around> to all get all the content (whether a filehandle if it's
+a regular file or a full string if it's a scalar ref) and decide what to do with
+it:
+
+    get '/download/:file' => sub {
+        return send_file(
+            params->{file},
+            streaming => 1,
+            callbacks => {
+                around => sub {
+                    my ( $writer, $content 0 = shift;
+
+                    # we know it's a text file, so we'll just stream
+                    # line by line
+                    while ( my $line = <$content> ) {
+                        $writer->write($line);
+                    }
+                },
+            },
+        );
+    }
+
+Or you could use C<override> to control the entire streaming callback request:
+
+    get '/download/:file' => sub {
+        return send_file(
+            params->{file},
+            streaming => 1,
+            callbacks => {
+                override => sub {
+                    my ( $respond, $response ) = @_;
+
+                    my $writer = $respond->( [ $newstatus, $newheaders ] );
+                    $writer->write("some line");
+                },
+            },
+        );
+    }
+
+You can also set the number of bytes that will be read at a time (default being
+42K bytes) using C<bytes>:
+
+    get '/download/:file' => sub {
+        return send_file(
+            params->{file},
+            streaming => 1,
+            bytes     => 524288, # 512K
+        );
+    };
+
+
 The content-type will be set depending on the current MIME types definition
 (see C<mime> if you want to define your own).
 
@@ -1448,6 +1694,20 @@ Tells the route handler to build a response with the current template engine:
         template 'some_view', { token => 'value'};
     };
 
+B<WARNING> : Issuing a template immediately exits the current route, and perform
+the template. Thus, any code after a template is ignored, until the end of the route.
+So it's not necessary anymore to use C<return> with template.
+
+    get '/some/route' => sub {
+        if (...) {
+            # we want to let the next matching route handler process this one
+            template(...);
+            # This code will be ignored
+            do_stuff();
+        }
+    };
+
+
 The first parameter should be a template available in the views directory, the
 second one (optional) is a HashRef of tokens to interpolate, and the third
 (again optional) is a HashRef of options.
@@ -1458,6 +1718,12 @@ For example, to disable the layout for a specific request:
         template 'index.tt', {}, { layout => undef };
     };
 
+Or to request a specific layout, of course:
+
+    get '/user' => sub {
+        template 'user.tt', {}, { layout => 'user' };
+    };
+
 Some tokens are automatically added to your template (C<perl_version>,
 C<dancer_version>, C<settings>, C<request>, C<params>, C<vars> and, if
 you have sessions enabled, C<session>).  Check
@@ -1554,19 +1820,24 @@ versions:
 
 =head2 var
 
-Defines a variable shared between filters and route handlers.
+Provides an accessor for variables shared between filters and route handlers.
+Given a key/value pair, it sets a variable:
 
     before sub {
         var foo => 42;
     };
 
-Route handlers and other filters will be able to read that variable with the
-C<vars> keyword.
+Later, route handlers and other filters will be able to read that variable:
+
+    get '/path' => sub {
+        my $foo = var 'foo';
+        ...
+    };
 
 =head2 vars
 
 Returns the HashRef of all shared variables set during the filter/route
-chain:
+chain with the C<var> keyword:
 
     get '/path' => sub {
         if (vars->{foo} eq 42) {
