@@ -9,8 +9,9 @@ use Dancer::Config;
 use Dancer::ModuleLoader;
 use Dancer::Route::Registry;
 use Dancer::Logger;
+use Dancer::Exception qw(:all);
 
-Dancer::App->attributes(qw(name prefix registry settings));
+Dancer::App->attributes(qw(name app_prefix prefix registry settings on_lexical_prefix));
 
 # singleton that saves any app created, we want unicity for app names
 my $_apps = {};
@@ -28,15 +29,36 @@ sub set_running_app {
     Dancer::App->current($app);
 }
 
+sub set_app_prefix {
+    my ($self, $prefix) = @_;
+    $self->app_prefix($prefix);
+    $self->prefix($prefix);
+}
+
 sub set_prefix {
     my ($self, $prefix, $cb) = @_;
-    croak "not a valid prefix: `$prefix', must start with a /"
+
+    undef $prefix if defined($prefix) and $prefix eq "/";
+
+    raise core_app => "not a valid prefix: `$prefix', must start with a /"
       if defined($prefix) && $prefix !~ /^\//;
+
+    my $app_prefix = defined $self->app_prefix ? $self->app_prefix : "";
     my $previous = Dancer::App->current->prefix;
-    Dancer::App->current->prefix($prefix);
+
+    $prefix ||= "";
+
+    if (Dancer::App->current->on_lexical_prefix) {
+        Dancer::App->current->prefix($previous.$prefix);
+    } else {
+        Dancer::App->current->prefix($app_prefix.$prefix);
+    }
+
     if (ref($cb) eq 'CODE') {
+        Dancer::App->current->on_lexical_prefix(1);
         eval { $cb->() };
         my $e = $@;
+        Dancer::App->current->on_lexical_prefix(0);
         Dancer::App->current->prefix($previous);
         die $e if $e;
     }
@@ -81,7 +103,7 @@ sub reload_apps {
 
 sub find_route_through_apps {
     my ($class, $request) = @_;
-    for my $app (Dancer::App->applications) {
+    for my $app (Dancer::App->current, Dancer::App->applications) {
         my $route = $app->find_route($request);
         if ($route) {
             Dancer::App->current($route->app);
@@ -134,7 +156,7 @@ sub init {
     my ($self) = @_;
     $self->name('main') unless defined $self->name;
 
-    croak "an app named '" . $self->name . "' already exists"
+    raise core_app => "an app named '" . $self->name . "' already exists"
       if exists $_apps->{$self->name};
 
     # default values for properties
