@@ -10,6 +10,7 @@ use Dancer::Template;
 use Dancer::ModuleLoader;
 use Dancer::FileUtils 'path';
 use Carp;
+use Dancer::Exception qw(:all);
 
 use Encode;
 
@@ -72,7 +73,7 @@ my $setters = {
     },
     traces => sub {
         my ($setting, $traces) = @_;
-        $Carp::Verbose = $traces ? 1 : 0;
+        $Dancer::Exception::Verbose = $traces ? 1 : 0;
     },
 };
 $setters->{log_path} = $setters->{log_file};
@@ -84,7 +85,7 @@ my $normalizers = {
           or return $charset;
         my $encoding = Encode::find_encoding($charset);
         defined $encoding
-          or croak "Charset defined in configuration is wrong : couldn't identify '$charset'";
+          or raise core_config => "Charset defined in configuration is wrong : couldn't identify '$charset'";
         my $name = $encoding->name;
         # Perl makes a distinction between the usual perl utf8, and the strict
         # utf8 charset. But we don't want to make this distinction
@@ -417,9 +418,17 @@ use Template Toolkit, add the following to C<config.yml>:
 The name of the layout to use when rendering view. Dancer will look for
 a matching template in the directory $views/layout.
 
+Your can override the default layout using the third argument of the
+C<template> keyword. Check C<Dancer> manpage for details.
+
 
 =head2 Logging, debugging and error handling
 
+=head3 import_warnings (boolean, default: enabled)
+
+If true, or not present, C<use warnings> will be in effect in scripts in which
+you import C<Dancer>.  Set to a false value to disable this.
+
 =head3 startup_info (boolean)
 
 If set to true, prints a banner at the server start with information such as
