@@ -2,7 +2,8 @@ package Dancer::Deprecation;
 
 use strict;
 use warnings;
-use Carp qw/croak carp/;
+use Carp;
+use Dancer::Exception qw(:all);
 
 sub deprecated {
     my ($class, %args) = @_;
@@ -25,7 +26,7 @@ sub deprecated {
     $msg .= " since version $deprecated_at" if defined $deprecated_at;
     $msg .= ". " . $args{reason} if defined $args{reason};
 
-    croak($msg) if $args{fatal};
+    raise core_deprecation => $msg if $args{fatal};
     carp($msg);
 }
 
@@ -59,7 +60,7 @@ List of possible parameters:
 
 =item B<message> message to display
 
-=item B<fatal> if set to true, croak instead of carp
+=item B<fatal> if set to true, raises a Dancer::Exception (Core::Deprecation) instead of carp
 
 =item B<reason> why is the feature deprecated
 
