@@ -8,6 +8,8 @@ use File::Spec;
 use Carp;
 use Cwd 'realpath';
 
+use Dancer::Exception qw(:all);
+
 use base 'Exporter';
 use vars '@EXPORT_OK';
 
@@ -65,7 +67,7 @@ sub open_file {
     my ( $mode, $filename ) = @_;
 
     open my $fh, $mode, $filename
-      or croak "$! while opening '$filename' using mode '$mode'";
+      or raise core_fileutils => "$! while opening '$filename' using mode '$mode'";
 
     return set_file_mode($fh);
 }
@@ -102,8 +104,7 @@ sub normalize_path {
     }x;
 
     $path =~ s{/\./}{/}g;
-    $path =~ s{$seqregex}{}g;
-    $path =~ s{$seqregex}{};
+    while ( $path =~ s{$seqregex}{} ) {}
 
     return $path;
 }
