@@ -60,13 +60,13 @@ sub format_message {
     my ($self, $level, $message) = @_;
     chomp $message;
 
-    if (setting('charset')) {
-        $message = Encode::encode(setting('charset'), $message);
+    if (my $charset = setting('charset')) {
+        $message = Encode::encode($charset, $message);
     }
 
     $level = 'warn' if $level eq 'warning';
     $level = sprintf('%5s', $level);
-    
+
     my $r     = Dancer::SharedData->request;
     my @stack = caller(3);
 
@@ -90,7 +90,7 @@ sub format_message {
               ? $r->env->{'HTTP_X_REAL_IP'} || $r->env->{'REMOTE_ADDR'}
               : '-';
         },
-        t => sub { Encode::decode(setting('charset'),
+        t => sub { Encode::decode(setting('charset') || 'utf8',
                                   POSIX::strftime( "%d/%b/%Y %H:%M:%S", localtime )) },
         T => sub { POSIX::strftime( "%Y-%m-%d %H:%M:%S", localtime  ) },
         P => sub { $$ },
