@@ -17,6 +17,7 @@ use Dancer::FileUtils qw(path path_or_empty dirname read_file_content open_file)
 use Dancer::SharedData;
 use Dancer::Logger;
 use Dancer::MIME;
+use Dancer::Exception qw(:all);
 
 Dancer::Factory::Hook->instance->install_hooks(
     qw/before after before_serializer after_serializer before_file_render after_file_render/
@@ -25,7 +26,8 @@ Dancer::Factory::Hook->instance->install_hooks(
 sub render_file { get_file_response() }
 
 sub render_action {
-    my $resp = get_action_response();
+    my $class = shift;
+    my $resp = $class->get_action_response();
     return (defined $resp)
       ? response_with_headers()
       : undef;
@@ -55,7 +57,9 @@ sub response_with_headers {
     my $response = Dancer::SharedData->response();
 
     $response->{headers} ||= HTTP::Headers->new;
-    $response->header('X-Powered-By' => "Perl Dancer ${Dancer::VERSION}");
+    my $powered_by = "Perl Dancer " . $Dancer::VERSION;
+    $response->header('X-Powered-By' => $powered_by);
+    $response->header('Server'       => $powered_by);
 
     return $response;
 }
@@ -78,6 +82,7 @@ sub html_page {
 }
 
 sub get_action_response {
+    my $class = shift;
     my $depth = shift || 1;
 
     # save the request before the filters are ran
@@ -99,19 +104,19 @@ sub get_action_response {
         || ($method ne Dancer::SharedData->request->method))
     {
         if ($depth > $MAX_RECURSIVE_LOOP) {
-            croak "infinite loop detected, "
+            raise core_renderer => "infinite loop detected, "
               . "check your route/filters for "
               . $method . ' '
               . $path;
         }
-        return get_action_response($depth + 1);
+        return $class->get_action_response($depth + 1);
     }
 
     # redirect immediately - skip route execution
     my $response = Dancer::SharedData->response();
     if (defined $response && (my $status = $response->status)) {
         if ($status == 302 || $status == 301) {
-            serialize_response_if_needed();
+            $class->serialize_response_if_needed();
             return $response;
         }
     }
@@ -119,11 +124,11 @@ sub get_action_response {
     # execute the action
     if ($handler) {
         # a response may exist, produced by a before filter
-        return serialize_response_if_needed() if defined $response && $response->exists;
+        return $class->serialize_response_if_needed() if defined $response && $response->exists;
         # else, get the route handler's response
         Dancer::App->current($handler->{app});
         $handler->run($request);
-        serialize_response_if_needed();
+        $class->serialize_response_if_needed();
         my $resp = Dancer::SharedData->response();
         Dancer::Factory::Hook->instance->execute_hooks('after', $resp);
         return $resp;
@@ -168,7 +173,6 @@ sub get_file_response {
 
 sub get_file_response_for_path {
     my ($class, $static_file, $status, $mime) = @_;
-    $status ||= 200;
 
     if ( -f $static_file ) {
         Dancer::Factory::Hook->execute_hooks( 'before_file_render',
@@ -177,8 +181,9 @@ sub get_file_response_for_path {
         my $fh = open_file( '<', $static_file );
         binmode $fh;
         my $response = Dancer::SharedData->response() || Dancer::Response->new();
-        $response->status($status);
+        $response->status($status) if ($status);
         $response->header('Content-Type' => (($mime && _get_full_mime_type($mime)) ||
+                                             Dancer::SharedData->request->content_type ||
                                              _get_mime_type($static_file)));
         $response->content($fh);
 
