@@ -4,7 +4,9 @@ use strict;
 use warnings;
 use Carp;
 use Dancer::ModuleLoader;
+use Dancer::Deprecation;
 use Dancer::Config 'setting';
+use Dancer::Exception qw(:all);
 use base 'Dancer::Serializer::Abstract';
 
 
@@ -26,7 +28,7 @@ sub loaded { Dancer::ModuleLoader->load('JSON') }
 
 sub init {
     my ($self) = @_;
-    croak 'JSON is needed and is not installed'
+    raise core_serializer => 'JSON is needed and is not installed'
       unless $self->loaded;
 }
 
@@ -74,8 +76,11 @@ sub _options_as_hashref {
         carp "options for to_json/from_json must be key value pairs (as a hashref)";
     }
     else {
-        carp "options as hash for to_json/from_json is DEPRECATED. please pass a hashref.";
-        return { @_ };
+        Dancer::Deprecation->deprecated(
+            version => '1.3002',
+            fatal => 1,
+            message => 'options as hash for to_json/from_json is DEPRECATED. please pass a hashref',
+        );
     }
 }
 
@@ -114,20 +119,16 @@ extra settings to the B<engines> configuration to turn these on.
             convert_blessed: '1'
 
 
-=head2 METHODS
+=head1 METHODS
 
-=over 4
-
-=item B<serialize>
+=head2 serialize
 
 Serialize a data structure to a JSON structure.
 
-=item B<deserialize>
+=head2 deserialize
 
 Deserialize a JSON structure to a data structure
 
-=item B<content_type>
+=head2 content_type
 
 Return 'application/json'
-
-=back
